@@ -24,7 +24,7 @@ use warnings;
 use autodie;
 
 use List::MoreUtils qw(any);
-use List::Util qw(first);
+use List::Util qw(first none);
 
 use Lintian::Data ();
 use Lintian::Relation ();
@@ -46,6 +46,8 @@ my $KNOWN_DBG_PACKAGE = Lintian::Data->new(
         return qr/$_[0]/xms;
     });
 
+my $SIGNING_KEY_FILENAMES = Lintian::Data->new('common/signing-key-filenames');
+
 sub run {
     my ($pkg, undef, $info, undef, $group) = @_;
     my $debian_dir = $info->index_resolved_path('debian/');
@@ -452,6 +454,42 @@ sub run {
           unless $info->source_field('xs-go-import-path', '');
     }
 
+    my $changes = $group->get_changes_processable;
+    tag 'source-only-upload-to-non-free-without-autobuild'
+      if defined($changes)
+      and $changes->info->field('architecture', '') eq 'source'
+      and $info->is_non_free
+      and $info->source_field('xs-autobuild', 'no') eq 'no';
+
+    # Ensure all orig tarballs have a signature if we have an upstream
+    # signature.
+    my $files = $info->files;
+    my $has_signing_key = 0;
+    for my $key_name ($SIGNING_KEY_FILENAMES->all) {
+        my $path = $info->index_resolved_path("debian/$key_name");
+        if ($path and $path->is_file) {
+            $has_signing_key = 1;
+            last;
+        }
+    }
+
+    # If we are using uscan in git tag mode, the signature will never
+    # match the tarball so pretend we don't have/need a signing key.
+    my $watch = $info->index_resolved_path('debian/watch');
+    if ($watch && $watch->file_contents =~ m/pgpmode=gittag/) {
+        $has_signing_key = 0;
+    }
+
+    foreach my $file (keys %$files) {
+        if (   $has_signing_key
+            && $file =~ m/(^.*\.orig(?:-[A-Za-z\d-]+)?\.tar)\./
+            && $file !~ m/\.asc$/
+            && !$info->repacked) {
+            tag 'orig-tarball-missing-upstream-signature', $file
+              if none { exists $files->{"$_.asc"} } ($file, $1);
+        }
+    }
+
     return;
 }
 
