@@ -49,7 +49,7 @@ my $compat_level = Lintian::Data->new('debhelper/compat-level',qr/=/);
 my $MISC_DEPENDS = Lintian::Relation->new('${misc:Depends}');
 
 sub run {
-    my (undef, undef, $info) = @_;
+    my (undef, undef, $info, undef, $group) = @_;
     my $droot = $info->index_resolved_path('debian/');
     my ($drules, $dh_bd_version, $level);
 
@@ -66,8 +66,11 @@ sub run {
     $bdepends = $info->relation('build-depends-all');
     my $seen_dh = 0;
     my $seen_dh_parallel = 0;
-    my $seen_python_helper = 0;
-    my $seen_python3_helper = 0;
+    my %seen = (
+        'python2' => 0,
+        'python3' => 0,
+        'runit'   => 0,
+    );
     my %overrides;
 
     $drules = $droot->child('rules') if $droot;
@@ -87,7 +90,7 @@ sub run {
         }
 
         next if /^\s*\#/;
-        if (m/^\s+-?(dh_\S+)/) {
+        if (m/^\s+[@+-]?(dh_\S+)/) {
             my $dhcommand = $1;
             $build_systems{'debhelper'} = 1
               if not exists($build_systems{'dh'});
@@ -101,11 +104,12 @@ sub run {
                   "$dhcommand (line $.)";
                 $uses_autotools_dev_dh = 1;
             }
-            if ($dhcommand eq 'dh_python3') {
-                $seen_python3_helper = 1;
-            }
-            if ($dhcommand =~ m,^dh_python(?:2$|\$.*),) {
-                $seen_python_helper = 1;
+
+            # Record if we've seen specific helpers, special-casing
+            # "dh_python" as Python 2.x.
+            $seen{'python2'} = 1 if $dhcommand eq 'dh_python2';
+            foreach my $k (keys %seen) {
+                $seen{$k} = 1 if $dhcommand eq "dh_$k";
             }
 
             if ($dhcommand eq 'dh_clean' and m/\s+\-k(?:\s+.*)?$/s) {
@@ -131,7 +135,7 @@ sub run {
             }
             $seencommand = 1;
             $needbuilddepends = 1;
-        } elsif (m,^\s+dh\s+,) {
+        } elsif (m,^\s+[@+-]?dh\s+,) {
             $build_systems{'dh'} = 1;
             delete($build_systems{'debhelper'});
             $seen_dh = 1;
@@ -159,20 +163,18 @@ sub run {
                     if (defined $depends) {
                         $missingbdeps_addons{$depends} = $addon;
                     }
-                    if ($addon eq 'python2') {
-                        $seen_python_helper = 1;
-                    } elsif ($addon eq 'python3') {
-                        $seen_python3_helper = 1;
+                    foreach my $k (keys %seen) {
+                        $seen{$k} = 1 if $addon eq $k;
                     }
                 }
             }
             if (m/--(after|before|until|remaining)/) {
                 tag 'dh-manual-sequence-control-obsolete', 'dh', $1;
             }
-            if (m,\$[({]\w,) {
-                # the variable could contain any add-ons
-                $seen_python_helper = 1;
-                $seen_python3_helper = 1;
+            # Variables could contain any add-ons so assume we have seen
+            # them all.
+            foreach my $addon (keys %seen) {
+                $seen{$addon} = 1 if m,\$[({]\w,;
             }
         } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,
             or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
@@ -229,6 +231,7 @@ sub run {
               }xsm
         ) {
             # All of these indirectly use dh.
+            $seencommand = 1;
             $build_systems{'dh'} = 1;
             delete($build_systems{'debhelper'});
         } elsif (
@@ -276,7 +279,17 @@ sub run {
             tag 'weak-dependency-on-misc-depends', $binpkg
               unless $strong->implies($MISC_DEPENDS);
         }
+    }
 
+    for my $proc ($group->get_processables('binary')) {
+        my $binpkg = $proc->pkg_name;
+        my $breaks = $info->binary_relation($binpkg, 'breaks');
+        my $strong = $info->binary_relation($binpkg, 'strong');
+        tag 'package-uses-dh-runit-but-lacks-breaks-substvar', $binpkg
+          if $seen{'runit'}
+          and $strong->implies('runit')
+          and any { m,^etc/sv/, } $proc->info->sorted_index
+          and not $breaks->implies('${runit:Breaks}');
     }
 
     my $compatnan = 0;
@@ -522,7 +535,7 @@ sub run {
         }
     }
 
-    if ($seen_dh and not $seen_python_helper) {
+    if ($seen_dh and not $seen{'python2'}) {
         my %python_depends;
         for my $binpkg (@pkgs) {
             if ($info->binary_relation($binpkg, 'all')
@@ -535,7 +548,7 @@ sub run {
               sort(keys %python_depends);
         }
     }
-    if ($seen_dh and not $seen_python3_helper) {
+    if ($seen_dh and not $seen{'python3'}) {
         my %python3_depends;
         for my $binpkg (@pkgs) {
             if ($info->binary_relation($binpkg, 'all')
