@@ -26,6 +26,7 @@ use warnings;
 use autodie;
 
 use Lintian::Tags qw(tag);
+use Lintian::Util qw($PKGREPACK_REGEX);
 
 our $WATCH_VERSION = Lintian::Data->new('watch-file/version', qr/\s*=\s*/o);
 our $SIGNING_KEY_FILENAMES= Lintian::Data->new('common/signing-key-filenames');
@@ -49,7 +50,7 @@ sub run {
     # source package sign, for fine grained version mangling check
     # If the version field is missing, we assume a neutral non-native one.
     my $version = $info->field('version', '0-1');
-    if ($version =~ /(dfsg|debian|ds|repack)/) {
+    if ($version =~ $PKGREPACK_REGEX) {
         $repack = $1;
     }
     if ($version =~ /(alpha|beta|rc)/i) {
@@ -61,7 +62,9 @@ sub run {
     my $fd = $wfile->open;
     local $_;
     while (<$fd>) {
-        $template = 1 if m/^\s*\#\s*Example watch control file for uscan/io;
+        $template = $.
+          if m/^\s*\#\s*Example watch control file for uscan/io
+          or m{<project>};
         next if /^\s*\#/;
         next if /^\s*$/;
         s/^\s*//;
@@ -95,10 +98,11 @@ sub run {
 
             my (
                 $repack_mangle, $repack_dmangle,
-                $prerelease_mangle, $prerelease_umangle
-            ) = (0, 0, 0, 0);
+                $repack_dmangle_auto,$prerelease_mangle,
+                $prerelease_umangle
+            ) = (0, 0, 0, 0, 0);
             my ($opts, @opts);
-            if (   s/^opt(?:ion)?s=\"([^\"]+)\"\s+//
+            if (   s/^opt(?:ion)?s=\"((?:[^\"]|\\\")+)\"\s+//
                 || s/^opt(?:ion)?s=(\S+)\s+//) {
                 $opts = $1;
                 @opts = split($watchver >= 4 ? '\s*,\s*' : ',', $opts);
@@ -115,6 +119,9 @@ sub run {
                     $prerelease_umangle = 1
                       if defined $prerelease
                       and /^uversionmangle\s*=.*$prerelease/;
+                    $repack_dmangle_auto = 1
+                      if $watchver >= 4
+                      and /^dversionmangle\s*=.*(?:s\/\@DEB_EXT\@\/|auto)/;
                     $withgpgverification = 1
                       if /^pgpsigurlmangle\s*=\s*/;
                     $withgpgverification = 1
@@ -135,7 +142,7 @@ sub run {
                               /project/showfiles\.php}xsm
                 or m{https?://(?:www\.)?(?:sourceforge|sf)\.net
                               /projects/.+/files}xsm
-              ) {
+            ) {
                 tag 'debian-watch-file-should-use-sf-redirector', "line $.";
             } elsif (m%((?:http|ftp):(?!//sf.net/)\S+)%) {
                 tag 'debian-watch-uses-insecure-uri',$1;
@@ -159,7 +166,8 @@ sub run {
             # If the version of the package contains dfsg, assume that it needs
             # to be mangled to get reasonable matches with upstream.
             if ($needs_repack_mangling and not $repack_mangle) {
-                tag 'debian-watch-file-should-mangle-version', "line $.";
+                tag 'debian-watch-file-should-mangle-version', "line $."
+                  unless $repack_dmangle_auto;
             }
             if (    $needs_repack_mangling
                 and $repack_mangle
@@ -182,8 +190,10 @@ sub run {
     }
     close($fd);
 
-    tag 'debian-watch-contains-dh_make-template' if ($template);
-    tag 'debian-watch-may-check-gpg-signature' unless ($withgpgverification);
+    tag 'debian-watch-contains-dh_make-template', "(line $template)"
+      if $template;
+    tag 'debian-watch-does-not-check-gpg-signature'
+      unless ($withgpgverification);
 
     # Look for upstream signing key
     my $key_found = 0;
